#!/usr/bin/env python3

import json
from typing import Annotated

from icalendar import Calendar, JCalParsingError
from rich.console import Console
import typer

console = Console()
err_console = Console(stderr=True)
ical2jcal = typer.Typer(add_completion=False)


def version_callback(value: bool):
    if value:
        import ical2jcal

        console.print(f"{ical2jcal.__name__}: {ical2jcal.__version__}")
        raise typer.Exit()


@ical2jcal.command()
def convert_to_jcal(
    ics_path: Annotated[typer.FileText, typer.Argument(exists=True)],
    jcal_path: Annotated[typer.FileTextWrite, typer.Argument()],
    pretty: Annotated[bool, typer.Option("--pretty", "-p", help="Pretty print json")] = False,
    version: Annotated[
        bool | None, typer.Option("--version", callback=version_callback, help="Print version")
    ] = None,
) -> None:
    """Convert an ics file to a jcal file.

    Convert a file from the RFC 5545 iCalendar format to the RFC 7265 jCalendar format.

    The file is expected to contain a single calendar.
    """
    try:
        calendars = Calendar.from_ical(ics_path.read(), multiple=True)
    except ValueError as e:
        raise typer.Exit("The input file is not a valid RFC 5545 iCalendar.") from e
    if len(calendars) == 0:
        raise typer.Exit("The input file is not a valid RFC 5545 iCalendar.")
    calendar = calendars[0]
    if pretty:
        console = Console(file=jcal_path)
        console.print_json(calendar.to_json())
    else:
        jcal_path.write(calendar.to_json())


jcal2ical = typer.Typer(add_completion=False)


@jcal2ical.command()
def convert_to_ical(
    jcal_path: Annotated[typer.FileText, typer.Argument(exists=True)],
    ics_path: Annotated[typer.FileBinaryWrite, typer.Argument()],
    version: Annotated[
        bool | None, typer.Option("--version", callback=version_callback, help="Print version")
    ] = None,
) -> None:
    """Convert an jcal file to an ics file.

    Convert a file from the RFC 7265 jCalendar format to the RFC 5545 iCalendar format.
    """
    try:
        calendar = Calendar.from_jcal(jcal_path.read())
    except json.JSONDecodeError as e:
        raise typer.Exit("The input file is not a valid RFC 7265 jCalendar.") from e
    except JCalParsingError as e:
        err_console.print(e)
        raise typer.Exit("The input file is not a valid RFC 7265 jCalendar.") from e
    ics_path.write(calendar.to_ical())



from pathlib import Path
from typing import NamedTuple

from icalendar import Calendar
import pytest

HERE = Path(__file__).parent
CALENDARS = HERE / "calendars"


class TestFile(NamedTuple):
    path: Path
    calendar: Calendar


@pytest.fixture
def calendars() -> dict[str, TestFile]:
    result = {}
    for path in CALENDARS.iterdir():
        if path.name.endswith(".ics"):
            calendar = Calendar.from_ical(path.read_text(), multiple=True)[0]
        else:
            calendar = Calendar.from_jcal(path.read_text())
        result[path.name] = TestFile(path, calendar)
    return result


@pytest.fixture
def output_file(tmp_path: Path) -> Path:
    return tmp_path / "output"



import json

import pytest
from typer.testing import CliRunner

from ical2jcal.cli import ical2jcal


def test_ical2jcal_single_calendar_file(calendars, output_file) -> None:
    """Convert ics to jcal using file arguments."""
    calendar = calendars["example.ics"]
    runner = CliRunner()
    result = runner.invoke(ical2jcal, [str(calendar.path), str(output_file)])
    assert result.exit_code == 0, result.output

    jcal = json.loads(output_file.read_text())
    expected = calendar.calendar.to_jcal()
    assert jcal == expected


def test_ical2jcal_single_calendar_stdin(calendars, output_file) -> None:
    """Convert ics to jcal using stdin."""
    calendar = calendars["example.ics"]
    runner = CliRunner()
    result = runner.invoke(ical2jcal, ["-", str(output_file)], input=calendar.path.read_text())
    assert result.exit_code == 0, result.output

    jcal = json.loads(output_file.read_text())
    expected = calendar.calendar.to_jcal()
    assert jcal == expected


def test_ical2jcal_single_calendar_stout(calendars) -> None:
    """Convert ics to jcal using stdin."""
    calendar = calendars["example.ics"]
    runner = CliRunner()
    result = runner.invoke(ical2jcal, [str(calendar.path), "-"])
    assert result.exit_code == 0, result.output

    jcal = json.loads(result.stdout)
    expected = calendar.calendar.to_jcal()
    assert jcal == expected


def test_pretty_option(calendars):
    """Test that we can print pretty json."""
    calendar = calendars["example.ics"]
    runner = CliRunner()
    result = runner.invoke(ical2jcal, [str(calendar.path), "-", "--pretty"])
    assert result.exit_code == 0, result.output

    jcal = json.loads(result.stdout)
    expected = calendar.calendar.to_jcal()
    assert jcal == expected
    assert result.output.startswith("[\n  ")


@pytest.mark.parametrize("content", ["", "Not a calendar"])
def test_error_input_is_not_a_calendar(tmp_path, content):
    """The input is not a calendar."""
    invalid_calendar = tmp_path / "invalid.ics"
    invalid_calendar.write_text(content)
    runner = CliRunner()
    result = runner.invoke(ical2jcal, [str(invalid_calendar), "-"])
    assert result.exit_code == 1, result.output
    assert "The input file is not a valid RFC 5545 iCalendar." in result.output


def test_error_multiple_calendars(calendars):
    """Multiple calendars exist."""
    calendar = calendars["stream.ics"]
    runner = CliRunner()
    result = runner.invoke(ical2jcal, [str(calendar.path), "-"])
    assert result.exit_code == 0, result.output

    jcal = json.loads(result.stdout)
    expected = calendar.calendar.to_jcal()
    assert jcal == expected



"""Command line package for converting iCalendar files to jCal files."""

import importlib.metadata

try:
    __version__ = importlib.metadata.version(__name__)
except importlib.metadata.PackageNotFoundError:
    __version__ = "0.0.0"  # Fallback for development mode



from nox import Session, options, param, parametrize
from nox_uv import session

options.error_on_external_run = True
options.default_venv_backend = "uv"
options.sessions = ["lint", "type_check", "test", "docs"]


@session(
    python=["3.10", "3.11", "3.12", "3.13", "3.14", "3.14t"],
    uv_groups=["test"],
)
def test(s: Session) -> None:
    s.run(
        "pytest",
        "--cov=ical2jcal",
        "--cov-report=html",
        "--cov-report=term",
        "--cov-fail-under=95",
        "tests",
        *s.posargs,
    )


# For some sessions, set venv_backend="none" to simply execute scripts within the existing outer
# uv-generated virtual environment, rather than have nox create a new one for each session. This
# makes commonly repeated sessions execute faster.
@session(venv_backend="none")
@parametrize(
    "command",
    [
        param(
            [
                "ruff",
                "check",
                ".",
                "--select",
                "I",
                # Also remove unused imports.
                "--select",
                "F401",
                "--extend-fixable",
                "F401",
                "--fix",
            ],
            id="sort_imports",
        ),
        param(["ruff", "format", "."], id="format"),
    ],
)
def fmt(s: Session, command: list[str]) -> None:
    s.run(*command)


@session(venv_backend="none")
@parametrize(
    "command",
    [
        param(["ruff", "check", "."], id="lint_check"),
        param(["ruff", "format", "--check", "."], id="format_check"),
    ],
)
def lint(s: Session, command: list[str]) -> None:
    s.run(*command)


@session(venv_backend="none")
def lint_fix(s: Session) -> None:
    s.run("ruff", "check", ".", "--extend-fixable", "F401", "--fix")


@session(venv_backend="none")
def type_check(s: Session) -> None:
    s.run("mypy", "src", "tests", "noxfile.py")


# Environment variable needed for mkdocstrings-python to locate source files.
doc_env = {"PYTHONPATH": "src"}


@session(venv_backend="none")
def docs(s: Session) -> None:
    s.run("mkdocs", "build", env=doc_env)


@session(venv_backend="none")
def docs_check_urls(s: Session) -> None:
    s.run("mkdocs", "build", env=doc_env | {"HTMLPROOFER_VALIDATE_EXTERNAL_URLS": str(True)})


@session(venv_backend="none")
def docs_offline(s: Session) -> None:
    s.run("mkdocs", "build", env=doc_env | {"MKDOCS_MATERIAL_OFFLINE": str(True)})


@session(venv_backend="none")
def docs_serve(s: Session) -> None:
    s.run("mkdocs", "serve", env=doc_env)


@session(venv_backend="none")
def docs_github_pages(s: Session) -> None:
    s.run("mkdocs", "gh-deploy", "--force", env=doc_env)


# Install only main dependencies for the license report.
@session(uv_groups=["licenses"], uv_no_install_project=True)
def licenses(s: Session) -> None:
    s.run(
        "pip-licenses",
        "--format=markdown",
        "--output-file=./docs/licenses/summary.txt",
        *s.posargs,
    )
    s.run(
        "pip-licenses",
        "--format=plain-vertical",
        "--with-license-files",
        "--no-license-path",
        "--output-file=./docs/licenses/license_files.txt",
    )
    s.run("pip-licenses", *s.posargs)
    s.run("pip-licenses", "--summary")



import pytest
from typer.testing import CliRunner

from ical2jcal import __version__, cli


@pytest.mark.parametrize("endpoint", [cli.ical2jcal, cli.jcal2ical])
def test_version(endpoint):
    runner = CliRunner()
    result = runner.invoke(endpoint, ["--version"])
    assert result.exit_code == 0, result.output
    assert "ical2jcal" in result.stdout
    assert __version__ in result.stdout



from icalendar import Calendar
import pytest
from typer.testing import CliRunner

from ical2jcal.cli import jcal2ical


def test_jcal2ical_single_calendar_file(calendars, output_file) -> None:
    """Convert ics to jcal using file arguments."""
    calendar = calendars["rfc_7265_appendix_example_1_jcal.jcal"]
    runner = CliRunner()
    result = runner.invoke(jcal2ical, [str(calendar.path), str(output_file)])
    assert result.exit_code == 0, result.output

    ics = output_file.read_text()
    assert Calendar.from_ical(ics) == calendar.calendar


@pytest.mark.parametrize("content", ["[]", "Not a calendar", ""])
def test_error_input_is_not_a_calendar(content, tmp_path):
    """The calendar is not valid."""
    invalid_calendar = tmp_path / "invalid.jcal"
    invalid_calendar.write_text(content)
    runner = CliRunner()
    result = runner.invoke(jcal2ical, [str(invalid_calendar), "-"])
    assert result.exit_code == 1, result.output
    assert "The input file is not a valid RFC 7265 jCalendar." in result.output


def test_error_jcal_information_problem_given(tmp_path):
    """The calendar is not valid."""
    invalid_calendar = tmp_path / "invalid.jcal"
    invalid_calendar.write_text("[]")
    runner = CliRunner()
    result = runner.invoke(jcal2ical, [str(invalid_calendar), "-"])
    assert result.exit_code == 1, result.output
    assert "in Calendar: A component must be a list with 3 items. Got value: []" in result.stderr
